#################################################################################
#'  Compute the posterior distribution of the approximation error between the true function
#'  and the estimates from variable subsets
#'
#' @param fdraws nd x n matrix where nd is the number of draws and n is the number of x used
#' @param faction n x p matrix, ith column is f estimate based on a subset evaluated at same x as used in fdraws
#' @param fbench n x 1, benchmark f estimate (e.g posterior mean using all variables)
#' @param distf function giving distance between two function evaluated at the x
#' @return list with components distbench: distance between f draws and benchmark, distaction: distances between f draws and f in faction
#'  ddistanction: difference between f is faction and benchmark f
#' @export
sumpost = function(fdraws,faction,fbench,distf) {###posterior summary using distf(ftrue,faction)
  #fdraws: nd x n matrix of draws from stage 1.  (i,j) is ith draw of f at jth future x
  #faction: n x p matrix of fhat using variable subsets. i,j is jth fhat evaluated at ith future x
  #fbench: n x 1, fhat used as benchmark (typically faction with all variables included, e.g. fbench = apply(fdraws,2,mean))
  #distf:  distance used: dist(fdraw,faction)

  #dimensions
  nd = nrow(fdraws) #number of draws
  n = ncol(fdraws) #number of future x
  p=ncol(faction) #number of action f
  cat("nd,n,p: ",nd,n,p,"\n")

  #storage
  distbench = rep(0,nd) #for each bart draw dist between draw f and fbench.
  distaction =matrix(0,nd,p) #same as above but for action f's.
  ddistaction =matrix(0,nd,p) #same as above but for differences

  #loop over draws in yhat.test
  for(i in 1:nd){
    distbench[i] = distf(fdraws[i,],fbench)
    for(j in 1:p) {
      distaction[i,j] = distf(fdraws[i,],faction[,j])
      ddistaction[i,j] = distaction[i,j]-distbench[i]
    }
  }
  resL = list(distbench=distbench,distaction=distaction,ddistaction=ddistaction)
  attr(resL, 'class') <- 'sumpost'
  return(resL)
}
######################################################################
#'  Root mean square error distance
#'
#' @param ftrue function possibly correct
#' @param faction function approximation (e.g based on a subset of variables)
#' @return distance (a number)
#' @export
distrmse = function(ftrue,faction) {
  return(sqrt(mean((ftrue-faction)^2)))
}
######################################################################
#' Mean absolute error
#'
#' @param ftrue Target function evaluated at a set of x
#' @param faction Approximating function evaluated at the same set of x
#' @return Mean absolute difference between function evaluations
#' @export
distabs = function(ftrue,faction) {
  return(mean(abs(ftrue-faction)))
}
########################################################################
#' Plot results of sumpost
#'
#' @param x Results from sumpost
#' @param diff If true subtract off benchmark distances
#' @param ylab label for y axis
#' @param xlab label for x axis
#' @param actnames names for the actions in the faction argument to sumpost.
#' @param pch plotting symbol for median passed onto ppdf.
#' @param hcol color used for horizontal line at 0 when diff=TRUE
#' @param cex.axis passed on to ppdf.
#' @param ... Passed on to ppdf
#' @export
plot.sumpost = function(x,diff=TRUE,ylab="distance",xlab="actions", actnames = 1:ncol(x$distaction),
                        pch=20,hcol="red",cex.axis=1,...) { ###plot sumpostGen results
  if(diff) {
    ddf = as.data.frame(x$ddistaction)
    names(ddf) = actnames
    ppdf(ddf,probs=c(.05,.95),ylim=range(c(x$ddistaction)),xlab=xlab,ylab=ylab,pch=pch,cex.axis=cex.axis,...)
    abline(h=0,col=hcol,lwd=2,lty=2)
  } else {
    distactiondf = as.data.frame(x$distaction)
    names(distactiondf) = actnames
    ppdf(distactiondf,probs=c(.05,.95),ylim=range(c(x$distaction,x$distbench)),xlab=xlab,ylab=ylab,pch=pch,cex.axis=cex.axis,...)
  }
}

