#' Plot forward and backward on the same plot
#'
#' @param vsfr  Forward results from vsf
#' @param vsbr Backwards results from vsb
#' @param ... passed on to plot(), title()
#' @export
plotfb = function(vsfr,vsbr,...) {
p=length(vsfr$R2)+1
R2f = vsfr$R2
R2b = vsbr$R2[(p-1):1]
plot(c(1,p-1),range(cbind(R2f,R2b)),xlab="iteration number",ylab="R-squared",...)
lines(1:(p-1),R2f,col="red",type="b",lty=2,pch=2,lwd=2)
lines(1:(p-1),R2b,col="blue",type="b",lty=3,pch=3,lwd=2)
legend("bottomright",legend=c("forwards","backwards"),col=c("red","blue"),lty=c(2,3),pch=c(2,3),lwd=c(2,2),
       bty="n",cex=1.2,y.intersp=1.2,seg.len=8)
title(main="variable susbset search, forward and backward",...)
}
#' all, forward and backward variable selection in the same plot.
#'
#' @param vsfr forward results from vsf
#' @param vsbr backward results from vsb
#' @param vsar all subsets results from vsa
#' @param ... passed on to plot(), title()
#' @export
plotfba = function(vsfr,vsbr,vsar,...) {
  p=length(vsfr$R2)+1
  R2f = vsfr$R2
  R2b = vsbr$R2[(p-1):1]
  R2a = vsar$R2
  plot(c(1,p-1),range(cbind(R2f,R2b,R2a)),xlab="number of variables",ylab="R-squared",...)
  lines(1:(p-1),R2f,col="red",type="b",lty=2,pch=2,lwd=2)
  lines(1:(p-1),R2b,col="blue",type="b",lty=3,pch=3,lwd=2)
  lines(1:(p-1),R2a,col="green",type="b",lty=4,pch=4,lwd=2)
  legend("bottomright",legend=c("forwards","backwards","all"),col=c("red","blue","green"),lty=c(2,3,4),pch=c(2,3,4),lwd=c(2,2),
         bty="n",cex=1.2,y.intersp=1.2,seg.len=8)
  title(main="variable susbset search, forward, backward, and all",...)
}
#' Run BART using various variable subsets in parallel using foreach
#'
#' @param X matrix of predictor variables, used as training data
#' @param y dependent variable, training data
#' @param Xp test data, returned value has BART predictions at these x values
#' @param vL list of variable subsets
#' @param ... passed on to BART
#' @return Matrix of BART fitted values, ith column for ith variable subset in vL
#' @export
bartSubs = function(X,y,Xp,vL,...) {### run bart on subsets vL
  if (!requireNamespace("BART", quietly = TRUE)) {
    stop("Package \"BART\" needed for this function to work. Please install it.",
         call. = FALSE)
  }
  if (!requireNamespace("foreach", quietly = TRUE)) {
    stop("Package \"foreach\" needed for this function to work. Please install it.",
         call. = FALSE)
  }
  i=1 #dummy to kill bartSubs: no visible binding for global variable ‘i’
  res = foreach(i=1:length(vL),.combine=cbind) %dopar% {
    vl = vL[[i]]
    if(length(vl)==1) {
      xs = as.matrix(as.double(X[,vl]),ncol=1)
      xps = as.matrix(as.double(Xp[,vl]),ncol=1)
      temp = BART::wbart(xs,y,xps,...)
    } else {
       Xs = X[,vL[[i]],drop=FALSE]
       Xps = Xp[,vL[[i]],drop=FALSE]
       temp = BART::wbart(Xs,y,Xps,...)
    }
    temp$yhat.test.mean
  }
  return(res)
}
###############################################################################
#' Plot the columns of a data frame using quantiles
#'
#' @param df The data frame to be plotted
#' @param probs The quantiles to to plot (just two)
#' @param linecol, color for quantile lines.
#' @param linetype line type for quantile lines.
#' @param cex.axis passed to axis.
#' @param ...  Passed on to plot and lines.
#' @export
ppdf = function(df,probs=c(.05,.95),linecol="black",linetype=1,cex.axis=1,...) {###quantile summarize columns
  nx = ncol(df)
  mx = unlist(lapply(df,median))
  qm = apply(df,2,quantile,probs=probs)
  yl = range(cbind(mx,t(qm)))

  plot(c(1,nx),yl,axes=FALSE,xlim=c(.8,nx+.2),type='n',...)
  axis(1,labels=names(df),at=1:nx,cex.axis=cex.axis)
  axis(2,cex.axis=cex.axis)

  for(i in 1:nx) {
    lines(c(i,i),qm[,i],col=linecol,lty=linetype)
  }
  points(1:nx,mx,...)
}


