#'  All subsets variable selection.
#'
#' @param X matrix of x values
#' @param fhat fitted values from nonlinear model and x vectors in xp
#' @param minbucket Minimum bucket size for big tree fit
#' @return List with components
#' vL: list of variable indices at each forward iteration step,
#' R2: R-squared at each step,
#' vrank: variable rankings,
#' xnames: variable names,
#' fit: matrix of fitted values
#' @export
vsa = function(X,fhat,minbucket=NA) {###all subsets variable selection

  p=ncol(X)
  n=nrow(X)
  if(is.na(minbucket)) minbucket=.5*log(n)
  M=2^p
  cat("doing M=2^p = ",M," models\n")

  if(is.null(colnames(X))) colnames(X) = paste("x",1:p,sep="")

  basetrees <- list()

  nvarv=rep(0,M)
  for(j in 2:M) {
    if((j %% 100) ==0) cat("j: ",j,", out of:", M,"\n")
    w <- as.integer(intToBits(j-1)[1:p])
    #cat("w:\n")
    #print(w)
    nvarv[j]=sum(w)
    Di = data.frame(cbind(X[,w==1,drop=FALSE],fhat))



    basetrees[[j]] <- rpart::rpart(fhat~.,data=Di,
                            control = rpart::rpart.control(minsplit = 2*minbucket,
                                                    minbucket=minbucket,
                                                    maxcompete=0,
                                                    maxsurrogate=0,xval=0,cp=0))
  }

  tsz = rep(0,M)
  rmsev = rep(0,M)
  R2    = rep(0,M)

  for(i in 2:M) {
    tsz[i] = length(unique(basetrees[[i]]$where))
    fitfit = predict(basetrees[[i]])
    rmsev[i] = sqrt(mean((fhat-fitfit)^2))
    R2[i] = cor(fhat,fitfit)^2
  }

  fitfit = matrix(0,n,p)
  ind = rep(0,p)
  bestR2 = rep(0,p)
  for(i in 1:p){
    ii = (1:M)[nvarv==i]
    ind[i] = ii[which.max(R2[ii])]
    fitfit[,i] =predict(basetrees[[ind[i]]])
    bestR2[i]=cor(fhat,fitfit[,i])^2
  }

  vL = vector("list",p-1)
  for(i in 1:(p-1)){
    j = ind[i]
    w <- as.integer(intToBits(j-1)[1:p])
    vL[[i]]=(1:p)[w==1]
  }
  R2=bestR2[-p]

  #vrank = vrankf(vL)

  #resL=list(vL=vL,R2=R2,vrank=vrank,xnames=colnames(xp))
  resL=list(vL=vL,R2=R2,xnames=colnames(X), fit = fitfit)

  attr(resL, 'class') <- 'vsa'
  return(resL)
}

#########################################################################
#' Prints the results of all subsets variable selection
#'
#' @param x Result from vsa.
#' @param ... not used.
#' @return List, ith element is the vector of variables names for the subset of size i.
#'    Note that unlike the the forward and backward case, the (i+1)th set of names man not be just the ith with one addition.
#' @export
print.vsa = function(x,...) {
  vLa = x$vL
  p = length(vLa)+1
  vLa[[p]]=1:p
  xLa = vector("list",p)
  for(i in 1:p) {
    xLa[[i]] = x$xnames[vLa[[i]]]
  }
  return(xLa)
}
######################################################################
#' Plot results of all subsets variable selection
#'
#' @param x output of vsf forward variable selection
#' @param maintext title for plot
#' @param plotvarnumber should variable number be plotted
#' @param lcol line color
#' @param pcol point color
#' @param ... passed on to plot(), text(), and axis()
#' @export
plot.vsa = function(x,maintext="all subsets variable selection",plotvarnumber=FALSE,lcol="blue",pcol="blue",...) {
  R2forward = x$R2
  nr2= length(R2forward)
  plot(1:nr2,R2forward,xlab="number of variables in subset",
       ylab="R-squared",main=maintext,type="n",axes=FALSE,...)
  lines(1:nr2,R2forward,col=lcol)
  if(plotvarnumber) {
    text(1:nr2,R2forward,col=pcol,x$vrank$varorder[1:nr2],...)
  } else {
    points(1:nr2,R2forward,col=pcol,...)
  }
  axis(1,1:nr2,labels=x$xnames[x$vrank$varorder[1:nr2]],...)
  axis(2,...)
  box()
}
