######################################################################
#' Backward nonlinear variable selection
#'
#' @param xp Matrix of x values
#' @param fhat fitted values from nonlinear model and x vectors in xp
#' @param minbucket Minimum bucket size for big tree fit
#' @return List with compontents vL, list of variable indices at each step,
#'   R2, R-squared at each step, vrank, variable rankings, xnames: variable names, fit: matrix of fitted values.
#' @export
vsb = function(xp,fhat,minbucket=NA) {
  p=ncol(xp)
  n=length(fhat)

  if(is.na(minbucket)) minbucket = .5*log(n)

  vL = list() #vL[[i]] is integer vector giving active vars
  vL[[1]] = 1:p #intilize to include all p vars

  G=TRUE
  itcount=1
  R2back=rep(0,p)
  fitL=list()

  while(G) {
    cat("in vsb, on iteration: " , itcount,"\n")

    temp = vL[[itcount]]
    nt = length(temp)
    temptreeL = vector("list",nt)
    for(j in 1:nt) {

      junk = temp[temp!=temp[j]]
      Di = data.frame(cbind(xp[,junk,drop=FALSE],fhat))
      temptreeL[[j]] <- rpart::rpart(fhat~.,data=Di,
                              control = rpart::rpart.control(minsplit = 2*minbucket,
                                                      minbucket=minbucket,
                                                      maxcompete=0,
                                                      maxsurrogate=0,xval=0,cp=0))
    }
    fitv = rep(0,nt)
    for(j in 1:nt) {
      fitv[j]=cor(predict(temptreeL[[j]]),fhat)^2
    }
    ii = which.max(fitv)
    itcount=itcount+1
    R2back[itcount]=fitv[ii]
    vL[[itcount]]= temp[temp!=temp[ii]]
    fitL[[itcount]] = predict(temptreeL[[ii]])
    if(itcount == (p)) G=FALSE
  }

  #drop initialization at vars=1:p
  nvs = length(vL)
  vL=vL[2:nvs]
  R2back=R2back[2:nvs]

  vrank = vrankb(vL)

  resL = list(vL=vL,R2=R2back,vrank=vrank,xnames=colnames(xp), fit = matrix(unlist(fitL[-1]),ncol=nvs-1))
  attr(resL, 'class') <- 'vsb'
  return(resL)
}
######################################################################
#' Computes the variable ranks from the vL component of the list returned by vsb.
#'
#' @param vL list of variables at each iteration from vsf results
#' @return list with components
#' rankofvar, the ith element is the the size of the smallest variable subset including variable i.
#' varorder, the order the variables come in, the ith element is the variable introduced to increase the subset size
#'   from (i-1) to i.
#' @export
vrankb = function(vL) { ###get which vars come in when from vsb results
  p = length(vL)+1
  avL = vector("list",p)
  avL[[p]]=1:p
  for(i in 1:(p-1)) {
    avL[[i]]=vL[[(p-1)-i+1]]
  }
  rankofvar = vector("integer",p) # rvec[i] is rank of variable i
  varofrank = vector("integer",p) # rvec[i] is rank of variable i
  rankofvar[avL[[1]][1]]=1
  varofrank[1]=avL[[1]][1]
  for(i in 2:p) {
    ii = setdiff(avL[[i]],avL[[i-1]])
    rankofvar[ii]=i
    varofrank[i]=ii
  }
  return(list(rankofvar=rankofvar,varorder=varofrank))
}
#########################################################################
#' Prints the results of backward variable selection
#'
#' @param x Result from vsb.
#' @param ... not used
#' @return Vector of R-squared values as the variables come in, labelled by the variable names.
#' @export
print.vsb = function(x,...) {
  p = length(x$R2)+1 #number of variables
  r2 = c(x$R2[(p-1):1],1.0)
  xnm = x$xnames
  vo = x$vrank$varorder
  names(r2) = xnm[vo]
  return(r2)
}
######################################################################
#' Plot results of backward variable selection
#'
#' @param x output of vsf forward variable selection
#' @param maintext title for plot
#' @param plotvarnumber should variable number be plotted
#' @param lcol line color
#' @param pcol point color
#' @param ... passed on to plot(), text(), points(), and  axis()
#' @export
plot.vsb = function(x,maintext="backward variable selection",plotvarnumber=FALSE,lcol="blue",pcol="blue",...) { ### do the R2 plot for from the results of vsf
  p = length(x$R2)+1 #number of variables
  R2forward = x$R2[(p-1):1]
  nr2= length(R2forward)
  plot(1:nr2,R2forward,xlab="",
       ylab="R-squared",main=maintext,type="n",axes=FALSE,...)
  lines(1:nr2,R2forward,col=lcol)
  if(plotvarnumber) {
    text(1:nr2,R2forward,col=pcol,x$vrank$varorder[1:nr2],...)
  } else {
    points(1:nr2,R2forward,col=pcol,...)
  }
  axis(1,1:nr2,labels=x$xnames[x$vrank$varorder[1:nr2]],las=3,...)
  axis(2,...)
  box()
}
