#####################################################################
#' Forward nonlinear variable selection
#'
#' @param xp Matrix of x values
#' @param fhat fitted values from nonlinear model and x vectors in xp
#' @param pMax Maximal number of variables
#' @param verbose Should messages be printed each time a variable is tried
#' @param minbucket Minimum bucket size for big tree fit
#' @return List with components
#' vL, list of variable indices at each forward iteration step,
#' R2, R-squared at each step,
#' vrank, variable rankings,
#' xnames, variable names
#' fit, matrix of fitted values
#' @export
vsf = function(xp,fhat,pMax=ncol(xp),verbose=FALSE,minbucket=NA) {###forwards variable selection
  p=ncol(xp)
  n=length(fhat)

  if(is.na(minbucket)) minbucket=floor(0.5*log(n))

  vL = list() #vL[[i]] is integer vector giving active vars

  p = dim(xp)[2]

  G=TRUE #while loop while this is true
  itcount=1
  R2forward=c()

  curvarlist=c()
  fitL=list()

  while(G) {
    cat("in vsf, on iteration: " , itcount,"\n")

    canvars = setdiff(1:p,curvarlist) #candidate variables
    nt = length(canvars)
    temptreeL = vector("list",nt)
    for(j in 1:nt) {

      if(verbose) cat("***j: ",j,"\n")

      junk = union(canvars[j],curvarlist)
      Di = data.frame(cbind(xp[,junk,drop=FALSE],fhat))
      temptreeL[[j]] <- rpart::rpart(fhat~.,data=Di,
                              control = rpart::rpart.control(minsplit = 2*minbucket,
                                                      minbucket=minbucket,
                                                      maxcompete=0,
                                                      maxsurrogate=0,xval=0,cp=0))
    }
    fitv = rep(0,nt)
    for(j in 1:nt) {
      fitv[j]=cor(predict(temptreeL[[j]]),fhat)^2
    }
    ii = which.max(fitv)
    fitL[[itcount]] = predict(temptreeL[[ii]])
    R2forward[itcount]=fitv[ii]
    vL[[itcount]]= union(curvarlist,canvars[ii])
    curvarlist = vL[[itcount]]
    itcount=itcount+1
    if((itcount == p) | (itcount > pMax)) G=FALSE
  }

  vrank = vrankf(vL)

  resL = list(vL=vL,R2=R2forward,vrank=vrank,xnames=colnames(xp),fit = matrix(unlist(fitL),ncol=length(fitL)))
  attr(resL, 'class') <- 'vsf'
  return(resL)
}
#########################################################################
#' Prints the results of forward variable selection
#'
#' @param x Result from vsf.
#' @param ... not used.
#' @return Vector of R-squared values as the variables come in, labelled by the variable names.
#' @export
print.vsf = function(x,...) {
  r2 = c(x$R2,1.0)
  xnm = x$xnames
  vo = x$vran$varorder
  names(r2) = xnm[vo]
  return(r2)
}
######################################################################
#' Plot results of forward variable selection
#'
#' @param x output of vsf forward variable selection
#' @param maintext title for plot
#' @param plotvarnumber should variable number be plotted
#' @param lcol line color
#' @param pcol point color
#' @param ... passed on to plot(), text(), points(), and axis()
#' @export
plot.vsf = function(x,maintext="forward variable selection",plotvarnumber=FALSE,lcol="blue",pcol="blue",...) {
  R2forward = x$R2
  nr2= length(R2forward)
  plot(1:nr2,R2forward,xlab="",
       ylab="R-squared",main=maintext,type="n",axes=FALSE,...)
  lines(1:nr2,R2forward,col=lcol)
  if(plotvarnumber) {
    text(1:nr2,R2forward,col=pcol,x$vrank$varorder[1:nr2],...)
  } else {
    points(1:nr2,R2forward,col=pcol,...)
  }
  axis(1,1:nr2,labels=x$xnames[x$vrank$varorder[1:nr2]],las=3,...)
  axis(2,...)
  box()
}
######################################################################
#' Computes the variable ranks from the vL component of the list returned by vsf.
#'
#' @param vL list of variables included at each iteration from vsf results
#' @return list with components
#' rankofvar, the ith element is the the size of the smallest variable subset including variable i.
#' varorder, the order the variables come in, the ith element is the variable introduced to increase the subset size
#'   from (i-1) to i.
#' @export
vrankf = function(vL) {
  ## rankofvar[i] = when var i comes in
  ## varofrank[i] = which var is ith in
  ## vL[[i] has list of variables 1 to (p-1) increasing in size
  p = length(vL)+1
  avL = vL; avL[[p]]=1:p
  rankofvar = vector("integer",p) # rvec[i] is rank of variable i
  varofrank = vector("integer",p) # rvec[i] is rank of variable i
  rankofvar[avL[[1]][1]]=1
  varofrank[1]=avL[[1]][1]
  for(i in 2:p) {
    ii = setdiff(avL[[i]],avL[[i-1]])
    rankofvar[ii]=i
    varofrank[i]=ii
  }
  return(list(rankofvar=rankofvar,varorder=varofrank))
}
